
const GObject = imports.gi.GObject;
const GLib = imports.gi.GLib;
const Lang = imports.lang;


const Values = new Lang.Class ({
  Name: 'Values',
  Extends: GObject.Object,
  Signals: {
    'change': {
      param_types: [GObject.TYPE_STRING, GObject.TYPE_VARIANT]
    }
  },
  
  /**
   * Name to keeyp track of what object we are using.
   */
  domain: "Non",
  
  /** @structure
   * Holds the holds of the tells that listners can listen to.
   */
  _tellers: {},
  
  /** @structure
   * Holds a list of lintners in a multi map:
   * <code>
   { "tell1": [callback0, callback1, callback2], "tell2": [callback4]}
   </code>
   */
  _listners: {},
  
  /** @structure
   * holds the definitions of values with their name, type, and optionally
   * user defined data that may be needed when dealing with getting and
   * setting of the values:
   * <code>
    {"value1": {type: "string", user_data: {any}},
     "value2": {type: "boolean", user_data: {any}}
    }
   </code>
   */
  _value_defs: {},
  
  /** @structure
   * Holds the data of the values in an encopsinated way, this makes it easyer
   * to deal with the values.
   * <code>
    {"value1": data, "value2": data}
   </code>
   */
  _values: {},
  
  
  /** @method
   * adds a setter to a value.
   *
   * @param name The name of the value to add a setter to.
   * @param func the function that is used for setting of data.
   *
   * @return true on succses
   * @return false on fail
   */
  add_setter: function (name, func) {
    if (name == undefined ||
        func == undefined ||
        this._value_defs[name] == undefined ||
        !(func instanceof Function)) {
      return false;
    }
    
  },
  
  _init: function (params) {
    this.parent (params);
    
    
    if (params != undefined) {
      if (params["domain"] != undefined) {
        this.domain = params["domain"];
      } else {
        print ("It is recomended that a Values object has a domain.");
      }
      
      if (params["tellers"] != undefined) {
        this._tellers = params["tellers"]
      }
    }
  },
  
  /** @method
   * Method to get the value stored.
   */
  get_value: function (name) {
    if (this._value_setters[name] == undefined) {
      print ("value \"" + name + "\" does not exist in the Values object.");
      return null; 
    }
    
    return _value_getters[name][getter] ();
  },
  
  /** @method
   * Sets a value.
   */
  set_value: function (name, value) {
    
    if (this._value_setters[name] == undefined) {
      print ("value \"" + name + "\" does not exist in the Values object.");
      return; 
    }
    
    if (this._value_setters[name]["type"] != typeof (value)) {
      print ("The value \"" + name + "\" is of the wrong type, should be a " +
      "\"" + this._value_setters[name]["type"] + "\".");
      return;
    }
    
    let ret_variant = null;
    
    switch (this._value_setters[name]["type"]) {
      case "number":
        ret_variant = GLib.Variant.new_int32 (value);
        break;
      case "boolean":
        ret_variant = GLib.Variant.new_boolean (value);
        break;
      case "string":
        ret_variant = GLib.Variant.new_string (value);
      default:
        print ("Balls!");
        return;
    }
    print ("Not balls!");
    this.emit ("change", name, ret_variant);
  }
  
});

main (ARGV);
